/*
 * MultiMAIL offline mail reader
 * 

   Written by Toth Istvan (stoty@vma.bme.hu)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "resource.h"
#include "misc.h"
#include <iostream.h>
#include <strings.h>
#include <stdlib.h>
#include <sys/stat.h>


resource::resource(mmail *mmA)
{
	mm = mmA;
	
	//set the names 
	const char *const names[noOfResources+1] = 
		{
			"",
			"homeDir",
			"mmHomeDir",
			"bwPacketDir",
			"bwWorkDir",
			"bwUncompressCommand",
			"bwPacketName",
			"bwUserName",
			"bwBBSName",
			"bwSysOpName",
			"bwReplyFileName",
			"bwEchoTag",
			"bwReplyDir",
			"bwLoginName",
			"bwAliasName",
			"bwCompressCommand",
			"bwUpWorkDir",
			"taglineChooseType",
			"sendConfirm",
			"editor",
			"mono",
			"bwSaveDir",
			"bwReadDir",
			"bwAddressBook",
			"bwTaglineFile"
		};
	
	// read these from rc 
	const int startUp[]=
			{
				homeDir,			
				mmHomeDir, 
				bwPacketDir,
				bwReplyDir,
				bwWorkDir,
				bwUpWorkDir,
				bwSaveDir,
				bwReadDir,
				bwCompressCommand,
				bwUncompressCommand,
				editor,
				taglineChooseType,
				sendConfirm,
				bwAddressBook,
				bwTaglineFile
			};

	// no of options in rc
	int startUpLen = 15; //edit !  
		
	char configFileName[255];
	initinit();
	
	// fill the name filed
	for(int c=1;c<=noOfResources;c++)
	{
		resourceData[c].name=(char*)names[c];
		resourceData[c].fromRc=0;   
	};

	// fill the fromrc field
	for(int c=0; c<startUpLen;c++)
	{
		resourceData[startUp[c]].fromRc=1;
	};

	strcpy(configFileName, getenv("HOME"));
	strcat(configFileName,"/.mmailrc");
	
	if((configFile=fopen(configFileName,"r")))
		parseConfig(configFile);
	for(int c=1;c<=noOfResources; c++)
		if( (resourceData[c].init) && (!resourceData[c].value) )
			(resourceData[c].init)();

};

int resource::parseConfig(FILE *configFile)
{
	int pos, p1, p2, p3;
	
	char buffer[255];
	char resName[255]; 
	char resValue[255];
	while(fgets(buffer, 255, configFile))
	{
		if((buffer[0]=='#') || (buffer[0]=='\n'))
			;
		else
		{
			pos=0;
			//trailing spaces
			while(buffer[pos]==' ' || buffer[pos]=='\t' && pos<255)
				pos++;
			p1=pos;
			//resName
			while(buffer[pos]!=':' && buffer[pos]!=' ' && buffer[pos]!='\t' && pos<255)
				resName[pos-p1]=buffer[pos++];
			//chars between strings
			while(buffer[pos]==' ' || buffer[pos]=='\t' || buffer[pos]==':' && pos<255)
				pos++;
			p2=pos;
			//resValue
			while(buffer[pos]!='\n' && pos<255)
				resValue[pos-p2]=buffer[pos++];
			p3=pos;
			//check	
			if(p1>=p2 || p2>=p3)
				fatalError("error parsing resource file");
			else
				for(int c=1; c<=noOfResources; c++)
					if(!strncmp(resourceData[c].name, resName, 255) && 
					(resourceData[c].fromRc))
						resourceData[c].value = strdupplus(resValue);
		};	
	};				
	return 0;
};

const char *const  resource::get(const int ID)
{
	if( (ID <=0) || (ID > noOfResources))
		return NULL;
	else
		return resourceData[ID].value;
};

const char* resource::set(const int ID, const char* newValue)
{
	if( (ID <=0) || (ID > noOfResources))
		return NULL;
	if(resourceData[ID].value)
		delete resourceData[ID].value;
	resourceData[ID].value = strdupplus(newValue);
	return newValue;
};

int resource::initinit()
{
	for(int c; c<=noOfResources; c++)
		(void*)resourceData[c].init = NULL;
		
	resourceData[homeDir].init=initHomeDir;
	resourceData[bwPacketDir].init=initBwPacketDir;
	resourceData[bwWorkDir].init=initBwWorkDir;
	resourceData[bwUncompressCommand].init=initBwUncompressCommand;
	resourceData[bwReplyFileName].init=initBwReplyFileName;
	resourceData[bwReplyDir].init=initBwReplyDir;
	resourceData[bwReadDir].init=initBwReadDir;
	resourceData[bwCompressCommand].init=initBwCompressCommand;
	resourceData[bwUpWorkDir].init=initBwUpWorkDir;
	resourceData[bwSaveDir].init=initBwSaveDir;
	resourceData[editor].init=initEditor;
	resourceData[mmHomeDir].init=initMmHomeDir;
	resourceData[bwAddressBook].init=initBwAddressBook;
	resourceData[bwTaglineFile].init=initBwTaglineFile;
	return 0;
};

//------------------------------------------------------------------------
// The resource initializer functions
//------------------------------------------------------------------------

int resource::initHomeDir()
{
	resourceData[homeDir].value=strdupplus(getenv("HOME"));
	return 0;
};	

// The defult is /tmp/$LOGNAME/bwwork
int resource::initBwWorkDir()
{
	char tmp[255];
	strcpy(tmp, "/tmp/");
	strcat(tmp, getenv("LOGNAME"));
	if((mkdir(tmp,S_IRWXU)))
		if( errno != EEXIST )
			fatalError("tmp Dir could not be created");
	strcat(tmp, "/bwwork");
	resourceData[bwWorkDir].value=strdupplus(tmp);
	if((mkdir(resourceData[bwWorkDir].value,S_IRWXU)))
		if( errno != EEXIST )
			fatalError("tmp Dir could not be created");
   return 0;
};

int resource::initBwUpWorkDir()
{
	char tmp[255];
	strcpy(tmp, "/tmp/");
	strcat(tmp, getenv("LOGNAME"));
	if((mkdir(tmp,S_IRWXU)))
		if( errno != EEXIST )
			fatalError("tmp Dir could not be created");
	strcat(tmp, "/bwupwork");
	resourceData[bwUpWorkDir].value=strdupplus(tmp);
	if((mkdir(resourceData[bwUpWorkDir].value,S_IRWXU)))
		if( errno != EEXIST )
	        fatalError("tmp Dir could not be created");
   return 0;
};

// The default is mmHomeDir/bwdown
int resource::initBwPacketDir()
{
	char    tmp[255];
	strcpy(tmp,resourceData[mmHomeDir].value);
	strcat(tmp,"/bwdown");
	resourceData[bwPacketDir].value = strdupplus(tmp);
	return 0;
};

// the default is mmHomeDir/bwread
int resource::initBwReadDir()
{
	char    tmp[255];
	strcpy(tmp,resourceData[mmHomeDir].value);
	strcat(tmp,"/bwread");
	resourceData[bwReadDir].value = strdupplus(tmp);
	return 0;
};


// the default is mmHomedir/bwup
int resource::initBwReplyDir()
{
	char    tmp[255];
	strcpy(tmp,resourceData[mmHomeDir].value);
	strcat(tmp,"/bwup");
	resourceData[bwReplyDir].value = strdupplus(tmp);
	return 0;
};

// the default is mmHomedir/bwsave
int resource::initBwSaveDir()
{
	char    tmp[255];
	strcpy(tmp,resourceData[mmHomeDir].value);
	strcat(tmp,"/bwsave/");
	resourceData[bwSaveDir].value = strdupplus(tmp);
	return 0;
};
                            
// The defult is infozip.
int resource::initBwUncompressCommand()
{
	resourceData[bwUncompressCommand].value = strdupplus("/usr/bin/unzip -o "); return 0;
};

int resource::initBwCompressCommand()
{
	resourceData[bwCompressCommand].value = strdupplus("/usr/bin/zip -o -j ");
	return 0;
};

//The default is "reply.txt"
int resource::initBwReplyFileName()
{
	resourceData[bwReplyFileName].value = strdupplus("reply.txt");
	return 0;
};

//The default is vi, or the EDITOR enviroment variable (a'la elm)
int resource::initEditor()
{
	if(getenv("EDITOR"))
		resourceData[editor].value=strdupplus(getenv("EDITOR"));
	else
		resourceData[editor].value=strdupplus("vi");	//argh!
	return 0;
};

// the default is ~/mmail
int resource::initMmHomeDir()
{
	char    tmp[255];
	strcpy(tmp,resourceData[homeDir].value);
	strcat(tmp,"/mmail");
	resourceData[mmHomeDir].value = strdupplus(tmp);
	return 0;
};

// the default is addressbook
int resource::initBwAddressBook()
{
	char tmp[255];
	strcpy(tmp, resourceData[mmHomeDir].value);
	strcat(tmp, "/addressbook");
	resourceData[bwAddressBook].value = strdupplus(tmp);
	return 0;
};

// the default is taglines
int resource::initBwTaglineFile()
{
	char tmp[255];
	strcpy(tmp, resourceData[mmHomeDir].value);
	strcat(tmp, "/taglines");
	resourceData[bwTaglineFile].value = strdupplus(tmp);
	return 0;
};
